"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JobRestoreAuth = void 0;
const FirebaseAuth_1 = require("../../utils/FirebaseAuth");
const template_1 = require("../../utils/template");
const stream_1 = require("stream");
const AuthConverter_1 = require("../../utils/AuthConverter");
const Logger_1 = require("../../utils/Logger");
class JobRestoreAuth extends template_1.JobBackupServiceRestoreTemplate {
    /**
     * @param settings - settings object
     * @param admin - firebase app
     * @param store - google cloud storage app
     */
    constructor(settings, admin, store) {
        super(settings, admin, store);
        this.auth = (0, FirebaseAuth_1.getAuth)(this.admin);
        const self = this;
        this.writeStream = new stream_1.Writable({
            write(object, encoding, callback) {
                (async () => {
                    if (object.service !== "auth" ||
                        typeof object.path !== "string" ||
                        typeof object.data !== "string")
                        return;
                    const userRef = object.path;
                    let userData;
                    if (self.settings.hash_config)
                        userData = AuthConverter_1.AuthConverter.fromString(object.data, true);
                    else
                        userData = AuthConverter_1.AuthConverter.fromString(object.data, false);
                    await self.writeBuffer.set(userRef, userData);
                    return;
                })()
                    .then(() => {
                    callback();
                    return;
                })
                    .catch((err) => {
                    callback(err);
                    return;
                });
            },
            objectMode: true,
        });
        this.writeStream.on("error", (err) => {
            Logger_1.Logger.warn(err);
        });
        this.writeBuffer = {
            batchSize: 1000,
            iteration: 0,
            batch: [],
            clear: async () => {
                self.writeBuffer.batch = [];
                return self.writeBuffer;
            },
            commit: async () => {
                let res;
                if (self.settings.hash_config)
                    res = await self.auth.importUsers(self.writeBuffer.batch, {
                        hash: self.settings.hash_config,
                    });
                else
                    res = await self.auth.importUsers(self.writeBuffer.batch);
                self.counter += res.successCount;
                if (res.failureCount !== 0)
                    Logger_1.Logger.warn(JSON.stringify(res.errors));
                await self.writeBuffer.clear();
                Logger_1.Logger.log(" -- Auth Restored - " +
                    self.counter +
                    " users in " +
                    self.getWorkTime() +
                    ".");
                return self.writeBuffer;
            },
            set: async (ref, data) => {
                self.writeBuffer.batch.push(data);
                ++self.writeBuffer.iteration;
                if (self.writeBuffer.iteration % self.writeBuffer.batchSize === 0) {
                    await self.writeBuffer.commit();
                }
                return self.writeBuffer;
            },
        };
    }
    /**
     * job runner
     */
    async run() {
        this.startTimestamp = Date.now();
        const self = this;
        const p = new Promise((res, rej) => {
            if (self.gunzipStream) {
                const gunzip = self.gunzipStream;
                self.gunzipStream.on("unpipe", () => {
                    gunzip.unpipe(self.parserStream);
                    gunzip.close();
                    self.parserStream.end();
                });
                self.fileStream
                    .pipe(gunzip)
                    .pipe(self.parserStream)
                    .pipe(self.writeStream);
            }
            else {
                self.fileStream.pipe(self.parserStream).pipe(self.writeStream);
            }
            self.writeStream.on("finish", () => {
                res();
            });
        });
        await p;
        if (this.writeBuffer.iteration % this.writeBuffer.batchSize !== 0)
            await this.writeBuffer.commit();
        await new Promise((res) => {
            setTimeout(res, 1);
        });
        Logger_1.Logger.log(" - Auth Restore Complete!");
        return;
    }
}
exports.JobRestoreAuth = JobRestoreAuth;
//# sourceMappingURL=auth.js.map